/*
Wrapper for EXT_framebuffer_object

Orion Sky Lawlor, olawlor@acm.org, 2005/12/06 (Public Domain)
*/
#ifndef __OGL_FRAMEBUFFER_H
#define __OGL_FRAMEBUFFER_H

#include "ogl/ext.h"
#include "ogl/util.h"

/** EXT_framebuffer_object wrapper class. 
  Has skeleton support for machines without EXT_framebuffer_object support,
  by doing buffer readback afterwards.
*/
class oglFramebuffer {
public:
	int w,h; ///< Size of our framebuffer
	int old_sz[4]; ///< Saved size of old framebuffer
	GLuint fb;///< Framebuffer object
	GLuint depth,stencil; ///< Renderbuffer for depth, stencil (or 0 if none)
	GLuint tex; ///< Texture objects to render into (or 0 if none)
	
	/// Make a new empty OpenGL texture with this internal format.
	GLuint make_tex(GLenum texFormat=GL_RGBA8);
	/// Make a new empty OpenGL renderbuffer with this format.
	GLuint make_rb(GLenum rbFormat=GL_DEPTH_COMPONENT24);
	
	oglFramebuffer(int wid,int ht,
		GLenum texFormat=GL_RGBA8,
		GLenum depthFormat=0, /* e.g., GL_DEPTH_COMPONENT24 */
		GLenum stencilFormat=0); /* e.g., GL_STENCIL_INDEX */
	~oglFramebuffer();
	
	/// Begin rendering to this framebuffer.  Does both bind and viewport.
	void bind(void);
	
	/// Finished rendering to this framebuffer.  Does unbind and restores viewpoint
	void unbind(void);
	
private:
	// Slower copy-framebuffer-to-texture version (not implemented)
	void readback(void);
};


#endif
